'' =================================================================================================
''
''   File....... jm_xbee_remote_at_reader.spin
''   Purpose.... Demonstrates remote AT command -- acquing values from remote radio
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2016 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 22 SEP 2017
''
'' =================================================================================================

{
  Jon's XBee setup using XCTU

  -- start with default configuration
  -- change baud (BD) to 57K
  -- change API enabled (AP) to 1 (no escapes)
  -- change DIO7 (D7) to disabled
  -- change DIO5 (D5) to disabled 
}


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal
                                                                 
  CLK_FREQ = _xinfreq * (_clkmode >> 6)                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us

  TRM_BAUD = 115_200                                            ' baud rate for terminal
  XB_BAUD  = xbee#BR_57K6                                       ' actual xbee baud
  
                                                                  
con { pab io pins }                                               
                                                                  
  RX1     = 31                                                  ' programming / terminal
  TX1     = 30                                      
                                                    
  SDA1    = 29                                                  ' eeprom / i2c
  SCL1    = 28                                                   
                                                                 
  AUD_RT  = 27                                                  ' audio (d/a) outputs
  AUD_LF  = 26                                                   
                                                                 
  LED_27  = 27                                                  ' leds (on audio outputs)
  LED_26  = 26                                                   
                                                                 
  SD_CS   = 25                                                  ' usd card
  SD_DI   = 24                                                   
  SD_CLK  = 23                                                   
  SD_DO   = 22                                                   
                                                                 
  ADC_CS  = 21                                                  ' adc input
  ADC_CLK = 20                                                   
  ADC_DO  = 19                                                   
  ADC_DI  = 18                                                   
                                                                 
  SVO_6   = 17                                                  ' servo headers
  SVO_5   = 16                                                   
  SVO_4   = 15                                                   
  SVO_3   = 14                                                   
  SVO_2   = 13                                                   
  SVO_1   = 12


con { app io }

  RX2     =  7                                                  ' xbee
  TX2     =  6    
                                                                 
                                                                 
obj                                                              
                                                                 
' main                                                          ' * master spin cog
  time : "jm_time_80"                                           '   delays/timing
  io   : "jm_io"                                                '   basic io
  xbee : "jm_xbee_api1"                                         ' * XBee via API1 interface                                                   
  term : "jm_fullduplexserial"                                  ' * serial for terminal
                                                                 
' * uses cog when loaded                                         


var

  long  t0
  long  clean
  
                                                                                 
pub main                                           
                                                                 
  setup

  term.rxflush                                                  ' clear kbd buffer
  term.rx                                                       ' press a key
  term.tx(term#CLS)                                             ' clear the terminal
  xbee.flush                                                    ' clear xbee buffer

  if (xbee.check_radio < 0)                                     ' ensure radio is present
    term.str(string("Error: No radio detected"))
  else
    t0 := cnt
    repeat
      get_remote($00)                                           ' request data from remote $00
      show_remote_response                                      ' display response
      waitcnt(t0 += CLK_FREQ)                                   ' poll every second

  repeat
    waitcnt(0)


dat

  RemoteAT              byte    $7E                             ' api header
                        byte    $00, $0F                        ' payload length (15)
                        byte    $17                             ' remote AT
                        byte    $01                             ' packet id
                        byte    $00, $00, $00, $00              ' 64-bit addr (serial #)
                        byte    $00, $00, $00, $00               
                        byte    $52, $00                        ' 16-bit addr
                        byte    $02                             ' apply changes
                        byte    $49, $53                        ' "IS" (queried sample)                             
                        byte    $F7                             ' checksum

  
pub get_remote(n)

'' Get digital and analog input from remote n

  RemoteAT[14] := n                                             ' select remote
  RemoteAT[18] := xbee.calc_checksum(@RemoteAT[3], RemoteAT[2]) ' update checksum

  xbee.tx_frame(@RemoteAT, 19) 


dat

  Report                byte    "===================", 13 
                        byte    " Remote AT Request ", 13
                        byte    "===================", 13   
                        byte    13
                        byte    "Header..........", 13 
                        byte    "Payload Count...", 13 
                        byte    "Frame Type......", 13 
                        byte    "Frame ID........", 13
                        byte    "64-bit Source...", 13
                        byte    13 
                        byte    "16-bit Source...", 13       
                        byte    "AT Command......", 13
                        byte    "Status..........", 13  
                        byte    "Samples.........", 13  
                        byte    "Channels Mask...", 13  
                        byte    "IO Values.......", 13  
                        byte    "Checksum........", 13
                        byte    13
                        byte    "Button..........", 13
                        byte    "Pot Setting.....", 13            
                        byte    0


pub show_remote_response | len, p_buf

  ifnot (clean)                                                 ' if we need to show report
    term.tx(term#CLS)                                           '  clear terminal  
    term.str(@Report)                                           '  show report page
    clean := true
  else
    term.tx(term#HOME)
    
  len := xbee.get_rx_frame(25)
  if (len <> 26)
    show_rx_frame(len)  
  else
    p_buf := xbee.rx_addr
    display_bytes(17,  4,  1, p_buf + 00)                       ' header
    display_bytes(17,  5,  2, p_buf + 01)                       ' payload bytes
    display_bytes(17,  6,  1, p_buf + 03)                       ' frame type
    display_bytes(17,  7,  1, p_buf + 04)                       ' frame id
    display_bytes(17,  8,  4, p_buf + 05)                       ' 64-bit address
    display_bytes(17,  9,  4, p_buf + 09)
    display_bytes(17, 10,  2, p_buf + 13)                       ' 16-bit address
    display_bytes(17, 11,  2, p_buf + 15)                       ' at command
    display_bytes(17, 12,  1, p_buf + 17)                       ' status
    display_bytes(17, 13,  1, p_buf + 18)                       ' samples
    display_bytes(17, 14,  2, p_buf + 19)                       ' channels mask
    display_bytes(17, 15,  4, p_buf + 21)                       ' io values
    display_bytes(17, 16,  1, p_buf + 25)                       ' checksum

    cursor_xy(17, 18)
    if (byte[p_buf + 22] == 0)                                  ' lsb of digital bits
      term.str(string("Not Pressed"))
    else
      term.str(string("Pressed    "))

    cursor_xy(17, 19)
    result := byte[p_buf + 23] << 8                             ' msb
    result |= byte[p_buf + 24]                                  ' lsb
    term.dec(result)
    term.txn(" ", 3)  


pub display_bytes(x, y, count, p_src)

'' Display count bytes in hex format
'' -- p_src is pointer to bytes
'' -- x/y defines PST screen position

  cursor_xy(x, y)
  repeat count
    term.hex(byte[p_src++], 2)
    term.tx(" ")   


pub cursor_xy(x, y)

'' Move PST cursor to column x, line y

  term.tx(term#crsr_xy)
  term.tx(x)
  term.tx(y)


pub show_rx_frame(len) : idx                                    ' for debugging

'' Display contents of XBee frame
'' -- used when developing/debugging

  term.tx(term#CLS)
  term.str(string("Unexpected response", 13, 13))  
  term.str(string("Bytes = "))
  term.dec(len)
  term.tx(13)

  repeat len 
    if (idx > 0) and ((idx // 8) == 0)                          ' 8 bytes per line
      term.tx(13)
    term.tx("$")
    term.hex(xbee.get_rx(idx), 2)                               ' display as hex
    term.tx(" ")
    ++idx

  term.txn(13, 2)

  clean := false

                     
pub setup                                                        
                                                                 
  time.start                                                    ' setup for timing/delays
                                                                 
  io.start(0, 0)                                                ' clear pins (this cog) 
                                                        
  term.start(RX1, TX1, %0000, TRM_BAUD)                         ' start serial for terminal *

  xbee.startx(RX2, TX2, XB_BAUD)                                ' start serial for xbee * 


dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}